package fr.univamu.sortvisualization;

import fr.univamu.sorting.*;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.canvas.Canvas;
import javafx.scene.chart.LineChart;
import javafx.scene.chart.XYChart;
import javafx.scene.layout.HBox;
import javafx.scene.paint.Color;

import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;
import java.util.function.Function;


public class Controller implements Initializable {

  // window dimensions
  static final int WIDTH = 1500;
  static final int HEIGHT = 1000;

  // TODO: remove the two following interfaces,
  //       import your own interfaces instead.
  private interface Sortable<T> {}
  private interface Sorter { void sort(); }

  private final List<Algorithm> algorithms =
      List.of(  // TODO: add algorithms here, as shown below
//          new Algorithm("Insertion sort", InsertionSort::new),
      );

  private Sortable<Integer> prepareSortable(String name) {
    // TODO: return a sortable, with decorations.
    //       Don't forget to make a copy of values: do not sort values itself!
    return null;
  }


  // The length of the lists to be sorted
  private static final int LIST_LENGTH = 100;

  // The list of values to be sorted.
  private final List<Integer> values = IntLists.shuffledRange(0, LIST_LENGTH-1);

  // This property will contain the date at which the experiment starts.
  // You may use it in prepareSortable, you should not modify it.
  private long startingTime = 0;

  // **********************************************
  // you should not change anything below this line
  // **********************************************

  @FXML
  private LineChart<Number,Number> comparisonChart;
  @FXML
  private LineChart<Number,Number> swapChart;
  @FXML
  private HBox visuals;

  private final List<Sorter> sortables = new ArrayList<>();

  private record Algorithm(String name, Function<Sortable<Integer>, Sorter> builder) {}

  @Override
  public void initialize(URL location, ResourceBundle resources) {
    startingTime = System.currentTimeMillis() + 1000;
    for (Algorithm algo : algorithms) {
      add(algo);
    }
    long wait = Math.max(0,startingTime - System.currentTimeMillis());
    try { Thread.sleep(wait); } catch (InterruptedException ignored) { }
    for (Sorter sort : sortables) {
      Thread thread = new Thread(sort::sort);
      thread.setDaemon(true);
      thread.start();
    }
  }

  private void add(Algorithm algo) {
    sortables.add(algo.builder().apply(prepareSortable(algo.name())));
  }


  private Canvas prepareNewCanvas(double width, double height) {
    Canvas canvas = new Canvas();
    canvas.setWidth(width);
    canvas.setHeight(height);
    visuals.getChildren().add(canvas);
    return canvas;
  }

  private TimeDataSeries addSwapSeries(String name) {
    return addChartSeries(name, swapChart);
  }

  private TimeDataSeries addComparisonSeries(String name) {
    return addChartSeries(name,comparisonChart);
  }

  private TimeDataSeries addChartSeries(
      String name,
      LineChart<Number,Number> chart
  ) {
    XYChart.Series<Number,Number> series = new XYChart.Series<>();
    series.setName(name);
    chart.getData().add(series);
    return new TimeDataSeries(series);
  }

  private Color getNextColor() {
    int i = sortables.size();
    return i < colors.size() ? colors.get(i) : Color.BLACK;
  }

  private final List<Color> colors =
      List.of(
          Color.valueOf("0xf3622d"),
          Color.valueOf("0xfba71b"),
          Color.valueOf("0x57b757"),
          Color.valueOf("0x41a9c9"),
          Color.valueOf("0x4258c9"),
          Color.valueOf("0x9a42c8"),
          Color.valueOf("0xc84164"),
          Color.valueOf("0x888888")
          );


}